from abc import ABC, abstractmethod
import os
import json
import importlib

from p2.store.client import Client
from p2.logging import Logger


class Reporter(ABC):
    """
    Base class for any reporter driver.
    Don't call this class direcly.
    Usage:
    from p2.tools.driver import get_driver

        driver = get_driver(
            module='p2.reporter',
            driver=<driver>,
            class_name='Reporter')

        driver.create_report_profile()
        rows = driver.prepare_report_rows()
        driver.store_report_rows(rows)
    """

    def __init__(self, tag:str=None):
        p2_store_url=os.getenv('P2_STORE_URL')
        self.p2_store_client = Client(p2_store_url)
        self.tag = tag

        log_obj = Logger(name=__name__)
        self.logger = log_obj.getLogger()
        self.config = self.get_config()

    def create_report_profile(self, **record):
        try:
            self.report_id = self.p2_store_client.add_returning_key(
                    table='reports',
                    key='id',
                    record=record)
            if self.report_id is None:
                self.logger.error("Error creating a report record")
                return None
            return self.report_id
        except Exception as e:
            self.logger.error("Something went wrong here...: {}".format(e))
            self.logger.error(traceback.format_exc())
            self.logger.error("End of plays for today!")
            return None

    def get_config(self) -> dict:
        rec = self.p2_store_client.select(
            table='config',
            filter={'component': self.__module__, 'tag': self.tag},
            fetch=['config']
            )
        if rec is not None and len(rec) == 1:
            return rec[0]['config']
        else:
            raise Exception(f"No config found for driver {self.__module__} and tag {self.tag}")


    @abstractmethod
    def prepare_report_rows(self, tag: str) -> list:
        """ This needs to be implemented by each driver..."""
        pass

    def store_report_rows(self, rows):
        try:
            for row in rows:
                record={
                    'report_id': self.report_id,
                    'body': json.dumps(row, indent=2)
                }
                ret = self.p2_store_client.add(
                        table='report_entries',
                        record=record
                        )
                if ret is None:
                    self.logger.error("Error adding a report entry")
                    return None
            return len(rows)
        except Exception as e:
            self.logger.error("Something went wrong here...: {}".format(e))
            self.logger.error(traceback.format_exc())
            self.logger.error("End of plays for today!")
            return None
